require 'spec_helper'

class TestTask < Hyrb::Tasks::Base
  def run(env)
    env[:tasks] ||= []
    env[:tasks] << self.class
  end
end

class Task1 < TestTask
end

class Task2 < TestTask
  depends Task1
end

class Task3 < TestTask
  depends Task2
end

class Task4 < TestTask
  depends Task2, Task1
end

class Task5 < TestTask
  depends Task2
end

class Task6 < TestTask
  depends Task2, Task5
end

class InvokerTask < TestTask
  depends Task4

  def run(env)
    super
    invoke(Task6)
  end
end

class Task7 < TestTask
  depends InvokerTask
end

class DoubleInvokerTask < TestTask
  depends Task6

  def run(env)
    super
    invoke(Task7)
  end
end

describe Hyrb::Pipeline do
  subject { Hyrb::Pipeline }

  describe "#next_tasks" do
    it "deals correctly with no deps" do
      subject.new(Task1).next_tasks.must_equal [Task1]
    end

    it "deals correctly with nested dependencies" do
      subject.new(Task2).next_tasks.must_equal [Task1, Task2]
      subject.new(Task3).next_tasks.must_equal [Task1, Task2, Task3]
    end

    it "deals correctly with multiple dependencies" do
      subject.new(Task4).next_tasks.must_equal [Task1, Task2, Task4]
    end
  end

  describe "#run" do
    it "runs the right tasks in the right order" do
      subject.new(Task1).run[:tasks].must_equal [Task1]
      subject.new(Task2).run[:tasks].must_equal [Task1, Task2]
      subject.new(Task3).run[:tasks].must_equal [Task1, Task2, Task3]
      subject.new(Task4).run[:tasks].must_equal [Task1, Task2, Task4]
    end
  end

  describe "#invoke" do
    it "runs the right tasks" do
      subject.new(Task7).run[:tasks].must_equal [Task1, Task2, Task4, InvokerTask, Task5, Task6, Task7]
    end

    it "deals with nested invocations" do
      subject.new(DoubleInvokerTask).run[:tasks].must_equal [Task1, Task2, Task5, Task6, DoubleInvokerTask, Task4, InvokerTask, Task7]
    end
  end
end